#
#  fcmp.py
#  
#  Function to compare double precision numbers
#  Uses the module fpconst, see its copyright
#  the fcmp is based on fcmp.c of gsl


from fpconst import *

TYPEFLOAT = type(1.0)

def fcmp( x1, x2, epsilon = 0.00000000001):
	"""Compare x1 and x2 floats (double precision),
	   with 10 default 10 digit precision, returns:
	    1 if x1 > x2
	   -1 if x1 < x2
	    0 if x1 ~=~ x2."""
	
	### Needs additional checks for infinity numbers etc.
	### Find exponent of largest absolute value 
	if (abs(x1) > abs(x2)):
		max = x1
	else:
		max = x2
	
	exponent = Exponent(max)
	
	### Form a neighborhood of size  2 * delta 
	if (exponent < 0):
		delta = epsilon * (1.0/(1 << -exponent))
	else:
		delta = epsilon * (1 << exponent)
	
	
	difference = x1 - x2

	if (difference > delta):       # x1 > x2 
		return 1
	else:
		if (difference < -delta):  # x1 < x2 
			return -1
		else:                      # -delta <= difference <= delta 
			return 0               # x1 ~=~ x2 
